@@ -21,11 +21,14 @@ This module handles logins and session state for the library.
 It's done for you automatically by the module when you make the
 first authenticated request. See `login()` for details.
 """
+from time import sleep
 from datetime import timezone, datetime
 
 import requests
 from packaging import version
 from requests import PreparedRequest
+import requests.utils
+
 
 try:
     from simplejson.errors import JSONDecodeError
@@ -39,16 +42,23 @@ import projectal
 def status():
     """Get runtime details of the Projectal server (with version number)."""
     _check_creds_or_fail()
-    response = requests.get(_build_url('/management/status'), verify=projectal.__verify)
+    response = requests.get(_build_url("/management/status"), verify=projectal.__verify)
+    if response.status_code == 429:
+        timeout_seconds = int(
+            # Wait for 60 seconds by default if header is missing for whatever reason
+            response.headers.get("X-Rate-Limit-Retry-After-Seconds", "60")
+        )
+        sleep(timeout_seconds + 1)
+        return status()
     return response.json()
 
 
 def _check_creds_or_fail():
     """Correctness check: can't proceed if no API details supplied."""
     if not projectal.api_base:
-        raise LoginException('Projectal URL (projectal.api_base) is not set')
+        raise LoginException("Projectal URL (projectal.api_base) is not set")
     if not projectal.api_username or not projectal.api_password:
-        raise LoginException('API credentials are missing')
+        raise LoginException("API credentials are missing")
 
 
 def _check_version_or_fail():
@@ -58,9 +68,9 @@ def _check_version_or_fail():
     of this API client, raise a ProjectalVersionException.
     """
     status = projectal.status()
-    if status['status'] != 'UP':
-        raise LoginException('Projectal server status check failed')
-    v = projectal.status()['version']
+    if status["status"] != "UP":
+        raise LoginException("Projectal server status check failed")
+    v = projectal.status()["version"]
     min = projectal.MIN_PROJECTAL_VERSION
     if version.parse(v) >= version.parse(min):
         return True
@@ -82,19 +92,25 @@ def login():
     """
     _check_version_or_fail()
 
-    payload = {
-        'username': projectal.api_username,
-        'password': projectal.api_password
-    }
+    payload = {"username": projectal.api_username, "password": projectal.api_password}
     if projectal.api_application_id:
-        payload['applicationId'] = projectal.api_application_id
-    response = requests.post(_build_url('/auth/login'), json=payload, verify=projectal.__verify)
+        payload["applicationId"] = projectal.api_application_id
+    response = requests.post(
+        _build_url("/auth/login"), json=payload, verify=projectal.__verify
+    )
     # Handle errors here
-    if response.status_code == 200 and response.json()['status'] == 'OK':
-        projectal.cookies = response.cookies
+    if response.status_code == 200 and response.json()["status"] == "OK":
+        projectal.cookies = requests.utils.dict_from_cookiejar(response.cookies)
         projectal.api_auth_details = auth_details()
         return True
-    raise LoginException('Check the API URL and your credentials')
+    if response.status_code == 429:
+        timeout_seconds = int(
+            # Wait for 60 seconds by default if header is missing for whatever reason
+            response.headers.get("X-Rate-Limit-Retry-After-Seconds", "60")
+        )
+        sleep(timeout_seconds + 1)
+        return login()
+    raise LoginException("Check the API URL and your credentials")
 
 
 def auth_details():
@@ -102,20 +118,20 @@ def auth_details():
     Returns some details about the currently logged-in user account,
     including all permissions available to it.
     """
-    return projectal.get('/api/user/details')
+    return projectal.get("/api/user/details")
 
 
 def permission_list():
     """
     Returns a list of all permissions that exist in Projectal.
     """
-    return projectal.get('/api/permission/list')
+    return projectal.get("/api/permission/list")
 
 
 def ldap_sync():
     """Initiate an on-demand user sync with the LDAP/AD server configured in your
     Projectal server settings. If not configured, returns a HTTP 405 error."""
-    return projectal.post('/api/ldap/sync', None)
+    return projectal.post("/api/ldap/sync", None)
 
 
 def query(payload):
@@ -123,7 +139,7 @@ def query(payload):
     Executes a query and returns the result. See the
     [Query API](https://projectal.com/docs/latest#tag/Query) for details.
     """
-    return projectal.post('/api/query/match', payload)
+    return projectal.post("/api/query/match", payload)
 
 
 def date_from_timestamp(date):
@@ -131,7 +147,7 @@ def date_from_timestamp(date):
     E.g., `1647561600000` returns `2022-03-18`."""
     if not date:
         return None
-    return str(datetime.utcfromtimestamp(int(date)/1000).date())
+    return str(datetime.utcfromtimestamp(int(date) / 1000).date())
 
 
 def timestamp_from_date(date):
@@ -139,35 +155,43 @@ def timestamp_from_date(date):
     E.g., `2022-03-18` returns `1647561600000`."""
     if not date:
         return None
-    return int(datetime.strptime(date, "%Y-%m-%d").replace(tzinfo=timezone.utc).timestamp() * 1000)
+    return int(
+        datetime.strptime(date, "%Y-%m-%d").replace(tzinfo=timezone.utc).timestamp()
+        * 1000
+    )
+
 
 def timestamp_from_datetime(date):
     """Returns a timestamp from a datetime string.
     E.g. `2022-03-18 17:00` returns `1647622800000`."""
     if not date:
         return None
-    return int(datetime.strptime(date, "%Y-%m-%d %H:%M")
-               .replace(tzinfo=timezone.utc).timestamp() * 1000)
+    return int(
+        datetime.strptime(date, "%Y-%m-%d %H:%M")
+        .replace(tzinfo=timezone.utc)
+        .timestamp()
+        * 1000
+    )
+
 
 def post(endpoint, payload=None, file=None, is_json=True):
     """HTTP POST to the Projectal server."""
-    return __request('post', endpoint, payload, file=file, is_json=is_json)
+    return __request("post", endpoint, payload, file=file, is_json=is_json)
 
 
 def get(endpoint, payload=None, is_json=True):
     """HTTP GET to the Projectal server."""
-    return __request('get', endpoint, payload, is_json=is_json)
+    return __request("get", endpoint, payload, is_json=is_json)
 
 
 def delete(endpoint, payload=None):
     """HTTP DELETE to the Projectal server."""
-    return __request('delete', endpoint, payload)
+    return __request("delete", endpoint, payload)
 
 
 def put(endpoint, payload=None, file=None, form=False):
     """HTTP PUT to the Projectal server."""
-    return __request(
-        'put', endpoint, payload, file=file, form=form)
+    return __request("put", endpoint, payload, file=file, form=form)
 
 
 def __request(method, endpoint, payload=None, file=None, form=False, is_json=True):
@@ -183,29 +207,45 @@ def __request(method, endpoint, payload=None, file=None, form=False, is_json=Tru
     fun = getattr(requests, method)
     kwargs = {}
     if file:
-        kwargs['files'] = file
-        kwargs['data'] = payload
+        kwargs["files"] = file
+        kwargs["data"] = payload
     elif form:
-        kwargs['data'] = payload
+        kwargs["data"] = payload
     else:
-        kwargs['json'] = payload
-    response = fun(_build_url(endpoint), cookies=projectal.cookies, verify=projectal.__verify, **kwargs)
+        kwargs["json"] = payload
+
+    response = fun(
+        _build_url(endpoint),
+        cookies=projectal.cookies,
+        verify=projectal.__verify,
+        **kwargs
+    )
 
     try:
         # Raise error for non-200 response
         response.raise_for_status()
-    except HTTPError:
-        # If the error is from an expired JWT we can retry it by
-        # clearing the cookie. (Login happens on next call).
-        try:
-            r = response.json()
-            if (r.get('status', None) == 'UNAUTHORIZED'
-                    or r.get('message', None) == 'anonymousUser'
-                    or r.get('error', None) == 'Unauthorized'):
-                projectal.cookies = None
-                return __request(method, endpoint, payload, file)
-        except JSONDecodeError:
-            pass
+    except HTTPError as err:
+        if err.response.status_code == 401:
+            # If the error is from an expired JWT we can retry it by
+            # clearing the cookie. (Login happens on next call).
+            try:
+                r = response.json()
+                if (
+                    r.get("status", None) == "UNAUTHORIZED"
+                    or r.get("message", None) == "anonymousUser"
+                    or r.get("error", None) == "Unauthorized"
+                ):
+                    projectal.cookies = None
+                    return __request(method, endpoint, payload, file)
+            except JSONDecodeError:
+                pass
+        if err.response.status_code == 429:
+            timeout_seconds = int(
+                # Wait for 60 seconds by default if header is missing for whatever reason
+                response.headers.get("X-Rate-Limit-Retry-After-Seconds", "60")
+            )
+            sleep(timeout_seconds + 1)
+            return __request(method, endpoint, payload, file)
         raise ProjectalException(response) from None
 
     # We will treat a partial success as failure - we cannot silently
@@ -214,6 +254,8 @@ def __request(method, endpoint, payload=None, file=None, form=False, is_json=Tru
         raise ProjectalException(response)
 
     if not is_json:
+        if response.cookies:
+            projectal.cookies = requests.utils.dict_from_cookiejar(response.cookies)
         return response
     try:
         payload = response.json()
@@ -221,15 +263,19 @@ def __request(method, endpoint, payload=None, file=None, form=False, is_json=Tru
         # does not match what we expect for the API endpoint.
         __maybe_fail_status(response, payload)
         # If we have a timestamp, record it for whoever is interested
-        if 'timestamp' in payload:
-            projectal.response_timestamp = payload['timestamp']
+        if "timestamp" in payload:
+            projectal.response_timestamp = payload["timestamp"]
         else:
             projectal.response_timestamp = None
 
         # If we have a 'jobCase', return the data it points to, which is
         # what the caller is after (saves them having to do it every time).
-        if 'jobCase' in payload:
-            return payload[payload['jobCase']]
+        if "jobCase" in payload:
+            if response.cookies:
+                projectal.cookies = requests.utils.dict_from_cookiejar(response.cookies)
+            return payload[payload["jobCase"]]
+        if response.cookies:
+            projectal.cookies = requests.utils.dict_from_cookiejar(response.cookies)
         return payload
     except JSONDecodeError:
         # API always responds with JSON. If not, it's an error
@@ -252,17 +298,18 @@ def __maybe_fail_status(response, payload):
     if response.status_code == 201:
         expected = "CREATED"
 
-    got = payload.get('status', expected)
+    got = payload.get("status", expected) if isinstance(payload, dict) else expected
     if expected == got:
         return True
-    m = "Unexpected response calling {}. Expected status: {}. Got: {}". \
-        format(response.url, expected, got)
+    m = "Unexpected response calling {}. Expected status: {}. Got: {}".format(
+        response.url, expected, got
+    )
     raise ProjectalException(response, m)
 
 
 def _build_url(endpoint):
     req = PreparedRequest()
-    url = projectal.api_base.rstrip('/') + endpoint
-    params = {'alias': projectal.api_alias}
+    url = projectal.api_base.rstrip("/") + endpoint
+    params = {"alias": projectal.api_alias}
     req.prepare_url(url, params)
     return req.url
