@@ -5,13 +5,22 @@ from projectal.entity import Entity
 from projectal.linkers import *
 
 
-class Location(Entity, CompanyLinker, CustomerLinker, ProjectLinker, StaffLinker,
-               NoteLinker, CalendarLinker, TagLinker):
+class Location(
+    Entity,
+    CompanyLinker,
+    CustomerLinker,
+    ProjectLinker,
+    StaffLinker,
+    NoteLinker,
+    CalendarLinker,
+    TagLinker,
+):
     """
     Implementation of the [Location](https://projectal.com/docs/latest/#tag/Location) API.
     """
-    _path = 'location'
-    _name = 'location'
+
+    _path = "location"
+    _name = "location"
     _links = [NoteLinker, CalendarLinker, TagLinker]
     _links_reverse = [CompanyLinker, CustomerLinker, ProjectLinker, StaffLinker]
 
@@ -23,15 +32,15 @@ class Location(Entity, CompanyLinker, CustomerLinker, ProjectLinker, StaffLinker
         `until`: End date in `yyyy-MM-dd` format
         """
         if begin:
-            begin = datetime.strptime(begin, '%Y-%m-%d').date()
+            begin = datetime.strptime(begin, "%Y-%m-%d").date()
         if until:
-            until = datetime.strptime(until, '%Y-%m-%d').date()
+            until = datetime.strptime(until, "%Y-%m-%d").date()
 
-        url = '/api/location/{}/calendar?'.format(self['uuId'])
+        url = "/api/location/{}/calendar?".format(self["uuId"])
         params = []
-        params.append('begin={}'.format(begin)) if begin else None
-        params.append('until={}'.format(until)) if until else None
-        url += '&'.join(params)
+        params.append("begin={}".format(begin)) if begin else None
+        params.append("until={}".format(until)) if until else None
+        url += "&".join(params)
 
         cals = api.get(url)
         cals = [projectal.Calendar(c) for c in cals]
