@@ -3,28 +3,47 @@ from projectal.linkers import *
 from projectal import api
 
 
-class Project(Entity,  LocationLinker, CustomerLinker, FileLinker, StageLinker,
-              RebateLinker, StageListLinker, CompanyLinker, NoteLinker, TagLinker,
-              TaskInProjectLinker):
+class Project(
+    Entity,
+    LocationLinker,
+    CustomerLinker,
+    FileLinker,
+    StageLinker,
+    RebateLinker,
+    StageListLinker,
+    CompanyLinker,
+    NoteLinker,
+    TagLinker,
+    TaskInProjectLinker,
+):
     """
     Implementation of the [Project](https://projectal.com/docs/latest/#tag/Project) API.
     """
-    _path = 'project'
-    _name = 'project'
-    _links = [LocationLinker, CustomerLinker, FileLinker, StageLinker,
-              RebateLinker, StageListLinker, NoteLinker, TagLinker,
-              TaskInProjectLinker]
+
+    _path = "project"
+    _name = "project"
+    _links = [
+        LocationLinker,
+        CustomerLinker,
+        FileLinker,
+        StageLinker,
+        RebateLinker,
+        StageListLinker,
+        NoteLinker,
+        TagLinker,
+        TaskInProjectLinker,
+    ]
     _links_reverse = [CompanyLinker]
 
     @classmethod
     def stage_order(cls, uuId, stages):
         """Reorder the Project's Stage links in a customer order."""
-        url = '/api/project/stage_list/order?holder={}'.format(uuId)
+        url = "/api/project/stage_list/order?holder={}".format(uuId)
         api.post(url, stages)
         return True
 
     @classmethod
-    def autoschedule(cls, project, mode='ASAP'):
+    def autoschedule(cls, project, mode="ASAP"):
         """
         Autoschedule the project.
 
@@ -32,7 +51,7 @@ class Project(Entity,  LocationLinker, CustomerLinker, FileLinker, StageLinker,
 
         `mode`: `ASAP` or `ALAP`
         """
-        url = '/api/project/schedule?mode={}'.format(mode)
+        url = "/api/project/schedule?mode={}".format(mode)
         api.post(url, [project])
         return True
 
@@ -43,11 +62,7 @@ class Project(Entity,  LocationLinker, CustomerLinker, FileLinker, StageLinker,
             "type": "msql",
             "start": 0,
             "limit": -1,
-            "holder": "{}".format(self['uuId']),
-            "select": [
-                [
-                    "PROJECT.TASK.uuId"
-                ]
-            ],
+            "holder": "{}".format(self["uuId"]),
+            "select": [["PROJECT.TASK.uuId"]],
         }
         return [t[0] for t in api.query(payload)]
