@@ -6,7 +6,8 @@ class Skill(Entity, StaffLinker, TaskLinker, TaskTemplateLinker, TagLinker):
     """
     Implementation of the [Skill](https://projectal.com/docs/latest/#tag/Skill) API.
     """
-    _path = 'skill'
-    _name = 'skill'
+
+    _path = "skill"
+    _name = "skill"
     _links = [TagLinker]  # TODO: note?
     _links_reverse = [StaffLinker, TaskLinker, TaskTemplateLinker]
