@@ -6,7 +6,8 @@ class Stage(Entity, ProjectLinker, TaskLinker, TagLinker):
     """
     Implementation of the [Stage](https://projectal.com/docs/latest/#tag/Stage) API.
     """
-    _path = 'stage'
-    _name = 'stage'
+
+    _path = "stage"
+    _name = "stage"
     _links = [TagLinker]
     _links_reverse = [ProjectLinker, TaskLinker]
