@@ -5,7 +5,8 @@ class Tag(Entity):
     """
     Implementation of the [Tag](https://projectal.com/docs/latest/#tag/Tag) API.
     """
-    _path = 'tag'
-    _name = 'tag'
+
+    _path = "tag"
+    _name = "tag"
     _links = []
     _links_reverse = []
