@@ -1,6 +1,7 @@
 """
 The base Entity class that all entities inherit from.
 """
+
 import copy
 import logging
 import sys
@@ -34,8 +35,8 @@ class Entity(dict):
     """
 
     #: Child classes must override these with their entity names
-    _path = 'entity'  # URL portion to api
-    _name = 'entity'
+    _path = "entity"  # URL portion to api
+    _name = "entity"
 
     # And to which entities they link to
     _links = []
@@ -54,8 +55,8 @@ class Entity(dict):
         self.update = self.__update
         self.delete = self.__delete
         self.history = self.__history
-        self.__old = copy.deepcopy(self)
         self.__type_links()
+        self.__old = copy.deepcopy(self)
 
     # ----- LINKING -----
 
@@ -76,26 +77,28 @@ class Entity(dict):
         The backend only offers one side of the relationship.
         """
         d = {
-            'name': cls._link_name,
-            'link_key': cls._link_key or cls._link_name + 'List',
-            'data_name': cls._link_data_name,
-            'type': cls._link_type,
-            'entity': cls._link_entity or cls._link_name.capitalize(),
-            'reverse': reverse
+            "name": cls._link_name,
+            "link_key": cls._link_key or cls._link_name + "List",
+            "data_name": cls._link_data_name,
+            "type": cls._link_type,
+            "entity": cls._link_entity or cls._link_name.capitalize(),
+            "reverse": reverse,
         }
-        self._link_def_by_key[d['link_key']] = d
-        self._link_def_by_name[d['name']] = d
+        self._link_def_by_key[d["link_key"]] = d
+        self._link_def_by_name[d["name"]] = d
 
     def _add_link(self, to_entity_name, to_link):
-        self._link(to_entity_name, to_link, 'add', batch_linking=False)
+        self._link(to_entity_name, to_link, "add", batch_linking=False)
 
     def _update_link(self, to_entity_name, to_link):
-        self._link(to_entity_name, to_link, 'update', batch_linking=False)
+        self._link(to_entity_name, to_link, "update", batch_linking=False)
 
     def _delete_link(self, to_entity_name, to_link):
-        self._link(to_entity_name, to_link, 'delete', batch_linking=False)
+        self._link(to_entity_name, to_link, "delete", batch_linking=False)
 
-    def _link(self, to_entity_name, to_link, operation, update_cache=True, batch_linking=True):
+    def _link(
+        self, to_entity_name, to_link, operation, update_cache=True, batch_linking=True
+    ):
         """
         `to_entity_name`: Destination entity name (e.g. 'staff')
 
@@ -114,16 +117,16 @@ class Entity(dict):
         """
 
         link_def = self._link_def_by_name[to_entity_name]
-        to_key = link_def['link_key']
+        to_key = link_def["link_key"]
 
-        if isinstance(to_link, dict) and link_def['type'] == list:
+        if isinstance(to_link, dict) and link_def["type"] == list:
             # Convert input dict to list when link type is a list (we allow linking to single entity for convenience)
             to_link = [to_link]
 
             # For cases where user passed in dict instead of Entity, we turn them into
             # Entity on their behalf.
             typed_list = []
-            target_cls = getattr(sys.modules['projectal.entities'], link_def['entity'])
+            target_cls = getattr(sys.modules["projectal.entities"], link_def["entity"])
             for link in to_link:
                 if not isinstance(link, target_cls):
                     typed_list.append(target_cls(link))
@@ -132,24 +135,36 @@ class Entity(dict):
             to_link = typed_list
         else:
             # For everything else, we expect types to match.
-            if not isinstance(to_link, link_def['type']):
-                raise api.UsageException('Expected link type to be {}. Got {}.'.format(link_def['type'], type(to_link)))
+            if not isinstance(to_link, link_def["type"]):
+                raise api.UsageException(
+                    "Expected link type to be {}. Got {}.".format(
+                        link_def["type"], type(to_link)
+                    )
+                )
 
         if not to_link:
             return
 
-        url = ''
+        url = ""
         payload = {}
         request_list = []
         # Is it a reverse linker? If so, invert the relationship
-        if link_def['reverse']:
+        if link_def["reverse"]:
             for link in to_link:
-                request_list.extend(link._link(self._name, self, operation, update_cache, batch_linking=batch_linking))
+                request_list.extend(
+                    link._link(
+                        self._name,
+                        self,
+                        operation,
+                        update_cache,
+                        batch_linking=batch_linking,
+                    )
+                )
         else:
             # Only keep UUID and the data attribute, if it has one
             def strip_payload(link):
-                single = {'uuId': link['uuId']}
-                data_name = link_def.get('data_name')
+                single = {"uuId": link["uuId"]}
+                data_name = link_def.get("data_name")
                 if data_name and data_name in link:
                     single[data_name] = copy.deepcopy(link[data_name])
                 return single
@@ -157,55 +172,56 @@ class Entity(dict):
             # If batch linking is enabled and the entity to link is a list of entities,
             # a separate request must be constructed for each one because the final composite
             # request permits only one input per call
-            url = '/api/{}/link/{}/{}'.format(self._path, to_entity_name, operation)
+            url = "/api/{}/link/{}/{}".format(self._path, to_entity_name, operation)
             to_link_payload = None
             if isinstance(to_link, list):
                 to_link_payload = []
                 for link in to_link:
                     if batch_linking:
-                        request_list.append({
-                            'method': "POST",
-                            'invoke': url,
-                            'body': {
-                                'uuId': self['uuId'],
-                                to_key: [strip_payload(link)],
+                        request_list.append(
+                            {
+                                "method": "POST",
+                                "invoke": url,
+                                "body": {
+                                    "uuId": self["uuId"],
+                                    to_key: [strip_payload(link)],
+                                },
                             }
-                        })
+                        )
                     else:
                         to_link_payload.append(strip_payload(link))
             if isinstance(to_link, dict):
                 if batch_linking:
-                    request_list.append({
-                        'method': "POST",
-                        'invoke': url,
-                        'body': {
-                            'uuId': self['uuId'],
-                            to_key: strip_payload(to_link),
+                    request_list.append(
+                        {
+                            "method": "POST",
+                            "invoke": url,
+                            "body": {
+                                "uuId": self["uuId"],
+                                to_key: strip_payload(to_link),
+                            },
                         }
-                    })
+                    )
                 else:
                     to_link_payload = strip_payload(to_link)
 
             if not batch_linking:
-                payload = {
-                    'uuId': self['uuId'],
-                    to_key: to_link_payload
-                }
+                payload = {"uuId": self["uuId"], to_key: to_link_payload}
                 api.post(url, payload=payload)
 
         if not update_cache:
             return request_list
 
         # Set the initial state if first add. We need the type to be set to correctly update the cache
-        if operation == 'add' and self.get(to_key, None) is None:
-            if link_def.get('type') == dict:
+        if operation == "add" and self.get(to_key, None) is None:
+            if link_def.get("type") == dict:
                 self[to_key] = {}
-            elif link_def.get('type') == list:
+            elif link_def.get("type") == list:
                 self[to_key] = []
 
         # Modify the entity object's cache of links to match the changes we pushed to the server.
         if isinstance(self.get(to_key, []), list):
-            if operation == 'add':
+            if operation == "add":
                 # Sometimes the backend doesn't return a list when it has none. Create it.
                 if to_key not in self:
                     self[to_key] = []
@@ -216,15 +232,15 @@ class Entity(dict):
                 for to_entity in to_link:
                     # Find it in original list
                     for i, old in enumerate(self.get(to_key, [])):
-                        if old['uuId'] == to_entity['uuId']:
-                            if operation == 'update':
+                        if old["uuId"] == to_entity["uuId"]:
+                            if operation == "update":
                                 self[to_key][i] = to_entity
-                            elif operation == 'delete':
+                            elif operation == "delete":
                                 del self[to_key][i]
         if isinstance(self.get(to_key, None), dict):
-            if operation in ['add', 'update']:
+            if operation in ["add", "update"]:
                 self[to_key] = to_link
-            elif operation == 'delete':
+            elif operation == "delete":
                 self[to_key] = None
 
         # Update the "old" record of the link on the entity to avoid
@@ -237,7 +253,14 @@ class Entity(dict):
     # -----
 
     @classmethod
-    def create(cls, entities, params=None, batch_linking=True):
+    def create(
+        cls,
+        entities,
+        params=None,
+        batch_linking=True,
+        disable_system_features=True,
+        enable_system_features_on_exit=True,
+    ):
         """
         Create one or more entities of the same type. The entity
         type is determined by the subclass calling this method.
@@ -282,7 +305,7 @@ class Entity(dict):
                 typed_list.append(e)
         e_list = typed_list
 
-        endpoint = '/api/{}/add'.format(cls._path)
+        endpoint = "/api/{}/add".format(cls._path)
         if params:
             endpoint += params
         if not e_list:
@@ -300,12 +323,12 @@ class Entity(dict):
 
         objects = []
         for i in range(0, len(payload), projectal.chunk_size_write):
-            chunk = payload[i:i + projectal.chunk_size_write]
-            orig_chunk = e_list[i:i + projectal.chunk_size_write]
+            chunk = payload[i : i + projectal.chunk_size_write]
+            orig_chunk = e_list[i : i + projectal.chunk_size_write]
             response = api.post(endpoint, chunk)
             # Put uuId from response into each input dict
             for e, o, orig in zip(chunk, response, orig_chunk):
-                orig['uuId'] = o['uuId']
+                orig["uuId"] = o["uuId"]
                 orig.__old = copy.deepcopy(orig)
                 # Delete links from the history in order to trigger a change on them after
                 for key in orig._link_def_by_key:
@@ -323,8 +346,26 @@ class Entity(dict):
 
         if len(link_request_batch) > 0 and batch_linking:
             for i in range(0, len(link_request_batch), 100):
-                chunk = link_request_batch[i:i + 100]
-                api.post('/api/composite', chunk)
+                chunk = link_request_batch[i : i + 100]
+                if disable_system_features:
+                    chunk = [
+                        {
+                            "note": "Disable Scheduling",
+                            "invoke": "PUT /api/system/features?entity=scheduling&action=DISABLE",
+                        },
+                        {
+                            "note": "Disable Macros",
+                            "invoke": "PUT /api/system/features?entity=macros&action=disable",
+                        },
+                    ] + chunk
+                if not enable_system_features_on_exit:
+                    chunk.append(
+                        {
+                            "note": "Exit script execution, and do not restore some disabled commands",
+                            "invoke": "PUT /api/system/features?entity=script&action=EXIT",
+                        }
+                    )
+                api.post("/api/composite", chunk)
 
         if not isinstance(entities, list):
             return objects[0]
@@ -336,15 +377,18 @@ class Entity(dict):
         error if the requested link is not valid for this Entity type."""
         link_set = set()
         if links is not None:
-            if isinstance(links, str) or not hasattr(links, '__iter__'):
-                raise projectal.UsageException("Parameter 'links' must be a list or None.")
+            if isinstance(links, str) or not hasattr(links, "__iter__"):
+                raise projectal.UsageException(
+                    "Parameter 'links' must be a list or None."
+                )
 
             defs = cls({})._link_def_by_name
             for link in links:
                 name = link.lower()
                 if name not in defs:
                     raise projectal.UsageException(
-                        "Link '{}' is invalid for {}".format(name, cls._name))
+                        "Link '{}' is invalid for {}".format(name, cls._name)
+                    )
                 link_set.add(name)
         return link_set
 
@@ -397,7 +441,7 @@ class Entity(dict):
 
         if isinstance(entities, str):
             # String input is a uuId
-            payload = [{'uuId': entities}]
+            payload = [{"uuId": entities}]
         elif isinstance(entities, dict):
             # Dict input needs to be a list
             payload = [entities]
@@ -405,7 +449,7 @@ class Entity(dict):
             # List input can be a list of uuIds or list of dicts
             # If uuIds (strings), convert to list of dicts
             if len(entities) > 0 and isinstance(entities[0], str):
-                payload = [{'uuId': uuId} for uuId in entities]
+                payload = [{"uuId": uuId} for uuId in entities]
             else:
                 # Already expected format
                 payload = entities
@@ -417,20 +461,20 @@ class Entity(dict):
             if not isinstance(deleted_at, int):
                 raise projectal.UsageException("deleted_at must be a number")
 
-        url = '/api/{}/get'.format(cls._path)
+        url = "/api/{}/get".format(cls._path)
         params = []
-        params.append('links={}'.format(','.join(links))) if links else None
-        params.append('epoch={}'.format(deleted_at - 1)) if deleted_at else None
+        params.append("links={}".format(",".join(links))) if links else None
+        params.append("epoch={}".format(deleted_at - 1)) if deleted_at else None
         if len(params) > 0:
-            url += '?' + '&'.join(params)
+            url += "?" + "&".join(params)
 
         # We only need to send over the uuIds
-        payload = [{'uuId': e['uuId']} for e in payload]
+        payload = [{"uuId": e["uuId"]} for e in payload]
         if not payload:
             return []
         objects = []
         for i in range(0, len(payload), projectal.chunk_size_read):
-            chunk = payload[i:i + projectal.chunk_size_read]
+            chunk = payload[i : i + projectal.chunk_size_read]
             dicts = api.post(url, chunk)
             for d in dicts:
                 obj = cls(d)
@@ -440,11 +484,11 @@ class Entity(dict):
                 # sometimes omitting links if no links exist.
                 for link_name in link_set:
                     link_def = obj._link_def_by_name[link_name]
-                    if link_def['link_key'] not in obj:
-                        if link_def['type'] == dict:
-                            obj.set_readonly(link_def['link_key'], None)
+                    if link_def["link_key"] not in obj:
+                        if link_def["type"] == dict:
+                            obj.set_readonly(link_def["link_key"], None)
                         else:
-                            obj.set_readonly(link_def['link_key'], link_def['type']())
+                            obj.set_readonly(link_def["link_key"], link_def["type"]())
                 objects.append(obj)
 
         if not isinstance(entities, list):
@@ -456,7 +500,13 @@ class Entity(dict):
         return super(Entity, self).get(*args, **kwargs)
 
     @classmethod
-    def update(cls, entities, batch_linking=True):
+    def update(
+        cls,
+        entities,
+        batch_linking=True,
+        disable_system_features=True,
+        enable_system_features_on_exit=True,
+    ):
         """
         Save one or more entities of the same type. The entity
         type is determined by the subclass calling this method.
@@ -486,6 +536,17 @@ class Entity(dict):
         else:
             e_list = entities
 
+        # allows for filtering of link keys
+        typed_list = []
+        for e in e_list:
+            if not isinstance(e, Entity):
+                new = cls({})
+                new.update(e)
+                typed_list.append(new)
+            else:
+                typed_list.append(e)
+        e_list = typed_list
+
         # Reduce the list to only modified entities and their modified fields.
         # Only do this to an Entity subclass - the consumer may have passed
         # in a dict of changes on their own.
@@ -495,14 +556,14 @@ class Entity(dict):
             if isinstance(e, Entity):
                 changes = e._changes_internal()
                 if changes:
-                    changes['uuId'] = e['uuId']
+                    changes["uuId"] = e["uuId"]
                     payload.append(changes)
             else:
                 payload.append(e)
         if payload:
             for i in range(0, len(payload), projectal.chunk_size_write):
-                chunk = payload[i:i + projectal.chunk_size_write]
-                api.put('/api/{}/update'.format(cls._path), chunk)
+                chunk = payload[i : i + projectal.chunk_size_write]
+                api.put("/api/{}/update".format(cls._path), chunk)
 
         # Detect and apply any link changes
         # if batch_linking is enabled, builds a list of link requests
@@ -516,8 +577,26 @@ class Entity(dict):
 
         if len(link_request_batch) > 0 and batch_linking:
             for i in range(0, len(link_request_batch), 100):
-                chunk = link_request_batch[i:i + 100]
-                api.post('/api/composite', chunk)
+                chunk = link_request_batch[i : i + 100]
+                if disable_system_features:
+                    chunk = [
+                        {
+                            "note": "Disable Scheduling",
+                            "invoke": "PUT /api/system/features?entity=scheduling&action=DISABLE",
+                        },
+                        {
+                            "note": "Disable Macros",
+                            "invoke": "PUT /api/system/features?entity=macros&action=disable",
+                        },
+                    ] + chunk
+                if not enable_system_features_on_exit:
+                    chunk.append(
+                        {
+                            "note": "Exit script execution, and do not restore some disabled commands",
+                            "invoke": "PUT /api/system/features?entity=script&action=EXIT",
+                        }
+                    )
+                api.post("/api/composite", chunk)
 
         return True
 
@@ -525,10 +604,17 @@ class Entity(dict):
         """Use the dict update for instances."""
         return super(Entity, self).update(*args, **kwargs)
 
-    def save(self):
+    def save(
+        self,
+        batch_linking=True,
+        disable_system_features=True,
+        enable_system_features_on_exit=True,
+    ):
         """Calls `update()` on this instance of the entity, saving
         it to the database."""
-        return self.__class__.update(self)
+        return self.__class__.update(
+            self, batch_linking, disable_system_features, enable_system_features_on_exit
+        )
 
     @classmethod
     def delete(cls, entities):
@@ -546,7 +632,7 @@ class Entity(dict):
         """
         if isinstance(entities, str):
             # String input is a uuId
-            payload = [{'uuId': entities}]
+            payload = [{"uuId": entities}]
         elif isinstance(entities, dict):
             # Dict input needs to be a list
             payload = [entities]
@@ -554,7 +640,7 @@ class Entity(dict):
             # List input can be a list of uuIds or list of dicts
             # If uuIds (strings), convert to list of dicts
             if len(entities) > 0 and isinstance(entities[0], str):
-                payload = [{'uuId': uuId} for uuId in entities]
+                payload = [{"uuId": uuId} for uuId in entities]
             else:
                 # Already expected format
                 payload = entities
@@ -563,12 +649,12 @@ class Entity(dict):
             payload = entities
 
         # We only need to send over the uuIds
-        payload = [{'uuId': e['uuId']} for e in payload]
+        payload = [{"uuId": e["uuId"]} for e in payload]
         if not payload:
             return True
         for i in range(0, len(payload), projectal.chunk_size_write):
-            chunk = payload[i:i + projectal.chunk_size_write]
-            api.delete('/api/{}/delete'.format(cls._path), chunk)
+            chunk = payload[i : i + projectal.chunk_size_write]
+            api.delete("/api/{}/delete".format(cls._path), chunk)
         return True
 
     def __delete(self):
@@ -582,12 +668,12 @@ class Entity(dict):
         Each entity has its own set of required values when cloning.
         Check the API documentation of that entity for details.
         """
-        url = '/api/{}/clone?reference={}'.format(self._path, self['uuId'])
+        url = "/api/{}/clone?reference={}".format(self._path, self["uuId"])
         response = api.post(url, entity)
-        return response['jobClue']['uuId']
+        return response["jobClue"]["uuId"]
 
     @classmethod
-    def history(cls, UUID, start=0, limit=-1, order='desc', epoch=None, event=None):
+    def history(cls, UUID, start=0, limit=-1, order="desc", epoch=None, event=None):
         """
         Returns an ordered list of all changes made to the entity.
 
@@ -604,19 +690,19 @@ class Entity(dict):
 
         `event`:
         """
-        url = '/api/{}/history?holder={}&'.format(cls._path, UUID)
+        url = "/api/{}/history?holder={}&".format(cls._path, UUID)
         params = []
-        params.append('start={}'.format(start))
-        params.append('limit={}'.format(limit))
-        params.append('order={}'.format(order))
-        params.append('epoch={}'.format(epoch)) if epoch else None
-        params.append('event={}'.format(event)) if event else None
-        url += '&'.join(params)
+        params.append("start={}".format(start))
+        params.append("limit={}".format(limit))
+        params.append("order={}".format(order))
+        params.append("epoch={}".format(epoch)) if epoch else None
+        params.append("event={}".format(event)) if event else None
+        url += "&".join(params)
         return api.get(url)
 
     def __history(self, **kwargs):
         """Get history of instance."""
-        return self.__class__.history(self['uuId'], **kwargs)
+        return self.__class__.history(self["uuId"], **kwargs)
 
     @classmethod
     def list(cls, expand=False, links=None):
@@ -632,10 +718,10 @@ class Entity(dict):
 
         payload = {
             "name": "List all entities of type {}".format(cls._name.upper()),
-            "type": "msql", "start": 0, "limit": -1,
-            "select": [
-                ["{}.uuId".format(cls._name.upper())]
-            ],
+            "type": "msql",
+            "start": 0,
+            "limit": -1,
+            "select": [["{}.uuId".format(cls._name.upper())]],
         }
         ids = api.query(payload)
         ids = [id[0] for id in ids]
@@ -705,7 +791,7 @@ class Entity(dict):
             return matches[0]
 
     @classmethod
-    def search(cls, fields=None, term='', case_sensitive=True, links=None):
+    def search(cls, fields=None, term="", case_sensitive=True, links=None):
         """Find entities that contain the text `term` within `fields`.
         `fields` is a list of field names to target in the search.
 
@@ -717,14 +803,14 @@ class Entity(dict):
         ```
         """
         filter = []
-        term = '(?{}).*{}.*'.format('' if case_sensitive else '?', term)
+        term = "(?{}).*{}.*".format("" if case_sensitive else "?", term)
         for field in fields:
             filter.append(["{}.{}".format(cls._name.upper(), field), "regex", term])
-        filter = ['_or_', filter]
+        filter = ["_or_", filter]
         return cls.query(filter, links)
 
     @classmethod
-    def query(cls, filter, links=None):
+    def query(cls, filter, links=None, timeout=30):
         """Run a query on this entity with the supplied filter.
 
         The query is already set up to target this entity type, and the
@@ -735,15 +821,27 @@ class Entity(dict):
         See [the filter documentation](https://projectal.com/docs/v1.1.1#section/Filter-section)
         for a detailed overview of the kinds of filters you can construct.
         """
-        payload = {
-            "name": "Python library entity query ({})".format(cls._name.upper()),
-            "type": "msql", "start": 0, "limit": -1,
-            "select": [
-                ["{}.uuId".format(cls._name.upper())]
-            ],
-            "filter": filter
-        }
-        ids = api.query(payload)
+        ids = []
+        request_completed = False
+        limit = projectal.query_chunk_size
+        start = 0
+        while not request_completed:
+            payload = {
+                "name": "Python library entity query ({})".format(cls._name.upper()),
+                "type": "msql",
+                "start": start,
+                "limit": limit,
+                "select": [["{}.uuId".format(cls._name.upper())]],
+                "filter": filter,
+                "timeout": timeout,
+            }
+            result = projectal.query(payload)
+            ids.extend(result)
+            if len(result) < limit:
+                request_completed = True
+            else:
+                start += limit
+
         ids = [id[0] for id in ids]
         if ids:
             return cls.get(ids, links=links)
@@ -751,24 +849,26 @@ class Entity(dict):
 
     def profile_get(self, key):
         """Get the profile (metadata) stored for this entity at `key`."""
-        return projectal.profile.get(key, self.__class__._name.lower(), self['uuId'])
+        return projectal.profile.get(key, self.__class__._name.lower(), self["uuId"])
 
     def profile_set(self, key, data):
         """Set the profile (metadata) stored for this entity at `key`. The contents
         of `data` will completely overwrite the existing data dictionary."""
-        return projectal.profile.set(key, self.__class__._name.lower(), self['uuId'], data)
+        return projectal.profile.set(
+            key, self.__class__._name.lower(), self["uuId"], data
+        )
 
     def __type_links(self):
         """Find links and turn their dicts into typed objects matching their Entity type."""
 
         for key, _def in self._link_def_by_key.items():
             if key in self:
-                cls = getattr(projectal, _def['entity'])
-                if _def['type'] == list:
+                cls = getattr(projectal, _def["entity"])
+                if _def["type"] == list:
                     as_obj = []
                     for link in self[key]:
                         as_obj.append(cls(link))
-                elif _def['type'] == dict:
+                elif _def["type"] == dict:
                     as_obj = cls(self[key])
                 else:
                     raise projectal.UsageException("Unexpected link type")
@@ -797,14 +897,14 @@ class Entity(dict):
                         changed[key] = changes
                         break
             elif key not in self.__old and self[key] is not None:
-                changed[key] = {'old': None, 'new': self[key]}
+                changed[key] = {"old": None, "new": self[key]}
             elif self.__old.get(key) != self[key]:
-                changed[key] = {'old': self.__old.get(key), 'new': self[key]}
+                changed[key] = {"old": self.__old.get(key), "new": self[key]}
         return changed
 
     def _changes_for_link_list(self, link_def, key):
         changes = self.__apply_list(link_def, report_only=True)
-        data_key = link_def['data_name']
+        data_key = link_def["data_name"]
 
         # For linked entities, we will only report their UUID, name (if it has one),
         # and the content of their data attribute (if it has one).
@@ -813,37 +913,37 @@ class Entity(dict):
             if isinstance(entities, dict):
                 entities = [entities]
             for e in entities:
-                fields = {'uuId': e['uuId']}
-                name = e.get('name')
+                fields = {"uuId": e["uuId"]}
+                name = e.get("name")
                 if name:
-                    fields['name'] = e['name']
+                    fields["name"] = e["name"]
                 if data_key and e[data_key]:
                     fields[data_key] = e[data_key]
                 slim.append(fields)
             return slim
 
         out = {
-            'added': get_slim_list(changes.get('add', [])),
-            'updated': [],
-            'removed': get_slim_list(changes.get('remove', [])),
+            "added": get_slim_list(changes.get("add", [])),
+            "updated": [],
+            "removed": get_slim_list(changes.get("remove", [])),
         }
 
-        updated = changes.get('update', [])
+        updated = changes.get("update", [])
         if updated:
             before_map = {}
             for entity in self.__old.get(key):
-                before_map[entity['uuId']] = entity
+                before_map[entity["uuId"]] = entity
 
             for entity in updated:
-                old_data = before_map[entity['uuId']][data_key]
+                old_data = before_map[entity["uuId"]][data_key]
                 new_data = entity[data_key]
                 diff = {}
                 for key in new_data.keys():
                     if key not in old_data and new_data[key] is not None:
-                        diff[key] = {'old': None, 'new': new_data[key]}
+                        diff[key] = {"old": None, "new": new_data[key]}
                     elif old_data.get(key) != new_data[key]:
-                        diff[key] = {'old': old_data.get(key), 'new': new_data[key]}
-                out['updated'].append({'uuId': entity['uuId'], data_key: diff})
+                        diff[key] = {"old": old_data.get(key), "new": new_data[key]}
+                out["updated"].append({"uuId": entity["uuId"], data_key: diff})
         return out
 
     def _changes_internal(self):
@@ -873,12 +973,11 @@ class Entity(dict):
 
     @staticmethod
     def __link_data_differs(have_link, want_link, data_key):
-
         if data_key:
-            if 'uuId' in have_link[data_key]:
-                del have_link[data_key]['uuId']
-            if 'uuId' in want_link[data_key]:
-                del want_link[data_key]['uuId']
+            if "uuId" in have_link[data_key]:
+                del have_link[data_key]["uuId"]
+            if "uuId" in want_link[data_key]:
+                del want_link[data_key]["uuId"]
             return have_link[data_key] != want_link[data_key]
 
         # Links without data never differ
@@ -894,14 +993,20 @@ class Entity(dict):
         find_list = []
         if not self._is_new:
             for link in self._link_def_by_key.values():
-                if link['link_key'] in self and link['name'] not in self._with_links:
-                    need.append(link['name'])
-                    find_list.append(link['link_key'])
+                if link["link_key"] in self and link["name"] not in self._with_links:
+                    need.append(link["name"])
+                    find_list.append(link["link_key"])
 
         if len(need):
-            logging.warning("Entity links were modified but entity not fetched with links. "
-                            "For better performance, include the links when getting the entity.")
-            logging.warning("Fetching {} again with missing links: {}".format(self._name.upper(), ','.join(need)))
+            logging.warning(
+                "Entity links were modified but entity not fetched with links. "
+                "For better performance, include the links when getting the entity."
+            )
+            logging.warning(
+                "Fetching {} again with missing links: {}".format(
+                    self._name.upper(), ",".join(need)
+                )
+            )
             new = self.__fetch(self, links=need)
             for _list in find_list:
                 self.__old[_list] = copy.deepcopy(new.get(_list, []))
@@ -927,45 +1032,57 @@ class Entity(dict):
         to_remove = []
         to_update = []
         should_only_have = set()
-        link_key = link_def['link_key']
+        link_key = link_def["link_key"]
 
-        if link_def['type'] == list:
+        if link_def["type"] == list:
             want_entities = self.get(link_key, [])
             have_entities = self.__old.get(link_key, [])
 
             if not isinstance(want_entities, list):
-                raise api.UsageException("Expecting '{}' to be {}. Found {} instead.".format(
-                    link_key, link_def['type'].__name__, type(want_entities).__name__))
+                raise api.UsageException(
+                    "Expecting '{}' to be {}. Found {} instead.".format(
+                        link_key,
+                        link_def["type"].__name__,
+                        type(want_entities).__name__,
+                    )
+                )
 
             for want_entity in want_entities:
-                if want_entity['uuId'] in should_only_have:
-                    raise api.UsageException("Duplicate {} in {}".format(link_def['name'], link_key))
-                should_only_have.add(want_entity['uuId'])
+                if want_entity["uuId"] in should_only_have:
+                    raise api.UsageException(
+                        "Duplicate {} in {}".format(link_def["name"], link_key)
+                    )
+                should_only_have.add(want_entity["uuId"])
                 have = False
                 for have_entity in have_entities:
-                    if have_entity['uuId'] == want_entity['uuId']:
+                    if have_entity["uuId"] == want_entity["uuId"]:
                         have = True
-                        data_name = link_def.get('data_name')
-                        if data_name and self.__link_data_differs(have_entity, want_entity, data_name):
+                        data_name = link_def.get("data_name")
+                        if data_name and self.__link_data_differs(
+                            have_entity, want_entity, data_name
+                        ):
                             to_update.append(want_entity)
                 if not have:
                     to_add.append(want_entity)
             for have_entity in have_entities:
-                if have_entity['uuId'] not in should_only_have:
+                if have_entity["uuId"] not in should_only_have:
                     to_remove.append(have_entity)
-        elif link_def['type'] == dict:
+        elif link_def["type"] == dict:
             # Note: dict type does not implement updates as we have no dict links
             # that support update (yet?).
             want_entity = self.get(link_key, None)
             have_entity = self.__old.get(link_key, None)
 
             if want_entity is not None and not isinstance(want_entity, dict):
-                raise api.UsageException("Expecting '{}' to be {}. Found {} instead.".format(
-                    link_key, link_def['type'].__name__, type(have_entity).__name__))
+                raise api.UsageException(
+                    "Expecting '{}' to be {}. Found {} instead.".format(
+                        link_key, link_def["type"].__name__, type(have_entity).__name__
+                    )
+                )
 
             if want_entity:
                 if have_entity:
-                    if want_entity['uuId'] != have_entity['uuId']:
+                    if want_entity["uuId"] != have_entity["uuId"]:
                         to_remove = have_entity
                         to_add = want_entity
                 else:
@@ -985,20 +1102,29 @@ class Entity(dict):
             request_list = []
             if to_remove:
                 delete_requests = self._link(
-                        link_def['name'], to_remove, 'delete',
-                        update_cache=False, batch_linking=batch_linking
+                    link_def["name"],
+                    to_remove,
+                    "delete",
+                    update_cache=False,
+                    batch_linking=batch_linking,
                 )
                 request_list.extend(delete_requests)
             if to_update:
                 update_requests = self._link(
-                    link_def['name'], to_update, 'update',
-                    update_cache=False, batch_linking=batch_linking
+                    link_def["name"],
+                    to_update,
+                    "update",
+                    update_cache=False,
+                    batch_linking=batch_linking,
                 )
                 request_list.extend(update_requests)
             if to_add:
                 add_requests = self._link(
-                    link_def['name'], to_add, 'add',
-                    update_cache=False, batch_linking=batch_linking
+                    link_def["name"],
+                    to_add,
+                    "add",
+                    update_cache=False,
+                    batch_linking=batch_linking,
                 )
                 request_list.extend(add_requests)
             self.__old[link_key] = copy.deepcopy(want_entities)
@@ -1006,16 +1132,17 @@ class Entity(dict):
         else:
             changes = {}
             if to_remove:
-                changes['remove'] = to_remove
+                changes["remove"] = to_remove
             if to_update:
-                changes['update'] = to_update
+                changes["update"] = to_update
             if to_add:
-                changes['add'] = to_add
+                changes["add"] = to_add
             return changes
 
     @classmethod
     def get_link_definitions(cls):
         return cls({})._link_def_by_name
+
     # --- ---
 
     def entity_name(self):
