@@ -14,30 +14,48 @@ task = projectal.Task.create(project, {
 ```
 """
 
+
 class Currency:
-    USD = "USD"
+    AED = "AED"
+    ARS = "ARS"
     AUD = "AUD"
-    EUR = "EUR"
-    PLN = "PLN"
-    MYR = "MYR"
-    INR = "INR"
-    IRR = "IRR"
-    JPY = "JPY"
-    GBP = "GBP"
+    BGN = "BGN"
+    BRL = "BRL"
     CAD = "CAD"
     CHF = "CHF"
+    CLP = "CLP"
     CNY = "CNY"
+    COP = "COP"
+    CZK = "CZK"
+    DKK = "DKK"
+    EUR = "EUR"
+    GBP = "GBP"
     HKD = "HKD"
-    NZD = "NZD"
-    SEK = "SEK"
+    HUF = "HUF"
+    IDR = "IDR"
+    ILS = "ILS"
+    INR = "INR"
+    JPY = "JPY"
     KRW = "KRW"
-    SGD = "SGD"
-    NOK = "NOK"
     MXN = "MXN"
+    MYR = "MYR"
+    NOK = "NOK"
+    NZD = "NZD"
+    PEN = "PEN"
+    PHP = "PHP"
+    PKR = "PKR"
+    PLN = "PLN"
+    RON = "RON"
     RUB = "RUB"
-    ZAR = "ZAR"
+    SAR = "SAR"
+    SEK = "SEK"
+    SGD = "SGD"
+    THB = "THB"
     TRY = "TRY"
-    BRL = "BRL"
+    TWD = "TWD"
+    UAH = "UAH"
+    USD = "USD"
+    ZAR = "ZAR"
 
 
 class TaskType:
@@ -78,7 +96,7 @@ class PayFrequency:
 
 class DateLimit:
     Min = "1970-01-01"
-    Max = "9999-12-31"
+    Max = "3000-01-01"
 
 
 class CompanyType:
@@ -88,6 +106,7 @@ class CompanyType:
     Partner = "Partner"
     Affiliate = "Affiliate"
     Office = "Office"
+    Division = "Division"
 
 
 class CalendarType:
@@ -113,3 +132,15 @@ class GanttLinkType:
     StartToStart = "Start_to_start"
     FinishToFinish = "Finish_to_finish"
     StartToFinish = "Start_to_finish"
+
+
+class ComplexityLevel:
+    Low = "Low"
+    Medium = "Medium"
+    High = "High"
+
+
+class PriorityLevel:
+    Low = "Low"
+    Normal = "Normal"
+    High = "High"
