@@ -8,12 +8,16 @@ except ImportError:
 
 class LoginException(Exception):
     """Failed to log in with the credentials provided."""
+
     pass
 
+
 class UnsupportedException(Exception):
     """The API endpoint is not available for this entity."""
+
     pass
 
+
 class ProjectalException(Exception):
     """
     The API request failed. This exception will extract the specific
@@ -25,6 +29,7 @@ class ProjectalException(Exception):
     the first failure is shown. A counter is included to show how
     many failed (i.e.: 1 of 25).
     """
+
     def __init__(self, response, reason_message=None):
         """
         `response`: the response object
@@ -53,29 +58,39 @@ class ProjectalException(Exception):
                 if reason_message:
                     self.message = reason_message
                 else:
-                    self.message = 'Unexpected response (API client error)'
+                    self.message = "Unexpected response (API client error)"
             except HTTPError:
                 pass
 
         if code == 400 and not self.json:
-            self.message = 'Client request error'
+            self.message = "Client request error"
         if code == 500:
-            self.message = 'Internal server error'
+            self.message = "Internal server error"
         if self.json and (code == 400 or code == 422 or code == 207):
-            if self.json.get('feedbackList', False):
-                self.feedback = [f for f in self.json.get('feedbackList', [])] or None
-                clue = [f['clue'] for f in self.json['feedbackList'] if f['clue'] not in ['OK', 'Created']]
-                hint = [f['hint'].replace('\'', '\"') for f in self.json['feedbackList'] if f['clue'] not in ['OK', 'Created']]
+            if self.json.get("feedbackList", False):
+                self.feedback = [f for f in self.json.get("feedbackList", [])] or None
+                clue = [
+                    f["clue"]
+                    for f in self.json["feedbackList"]
+                    if f["clue"] not in ["OK", "Created"]
+                ]
+                hint = [
+                    f["hint"].replace("'", '"')
+                    for f in self.json["feedbackList"]
+                    if f["clue"] not in ["OK", "Created"]
+                ]
                 if len(clue) == 1:
                     clue_part = " - Clue: {} ({})".format(clue[0], hint[0])
                 else:
-                    clue_part = " - Clue 1 of {}: {} ({})".format(len(clue), clue[0], hint[0])
+                    clue_part = " - Clue 1 of {}: {} ({})".format(
+                        len(clue), clue[0], hint[0]
+                    )
             else:
                 # No feedback list, but clue in top-leel of dict
-                clue = self.json['jobClue']['clue']
-                hint = self.json['jobClue']['hint'].replace('\'', '\"')
-                clue_part = ' - Clue: {} ({})'.format(clue, hint)
-            self.message = self.json['status'] + clue_part
+                clue = self.json["jobClue"]["clue"]
+                hint = self.json["jobClue"]["hint"].replace("'", '"')
+                clue_part = " - Clue: {} ({})".format(clue, hint)
+            self.message = self.json["status"] + clue_part
 
         if not self.message:
             self.message = "Request error: {}".format(code)
@@ -87,6 +102,7 @@ class ProjectalVersionException(Exception):
     The Projectal server version is incompatible with the version
     of this library.
     """
+
     pass
 
 
@@ -95,4 +111,5 @@ class UsageException(Exception):
     The library was used incorrectly and we were able to detect it
     before making a request.
     """
-    pass
\ No newline at end of file
+
+    pass
